@@ -139,7 +139,7 @@ impl Environment {
         }
     }
 
-    pub fn declare_sort(&mut self, sortname: &str) {
+    fn declare_sort(&mut self, sortname: &str) {
         if !self.context.sortid.contains_key(sortname) {
             self.context
                 .sortid
@@ -232,7 +232,10 @@ impl Environment {
             }
         }
         let mut tokens = tokens.into_iter();
-        let name = tokens.next().unwrap();
+        let name = match tokens.next() {
+            Some(s) => s,
+            None => return Err(Error::IllegalTerm(String::from(s))),
+        };
         let mut args = Vec::new();
         for s in tokens {
             args.push(self.read_term(s.as_str(), variables)?);
@@ -345,29 +348,8 @@ impl Environment {
 impl<'a> ToString for Environment {
     fn to_string(&self) -> String {
         let mut s = String::new();
-        for (name, id) in self.context.sortid.iter() {
-            s.push_str(&format!("(declare-sort {} {}))", name, id));
-        }
-        for (name, id) in self.context.functionid.iter() {
-            s.push_str(&format!(
-                "(declare-fun {} ({}) {})",
-                name,
-                self.context
-                    .argsorts
-                    .get(id)
-                    .unwrap()
-                    .iter()
-                    .map(|x| self.context.sortname.get(x).unwrap().clone())
-                    .collect::<Vec<String>>()
-                    .join(" "),
-                self.context
-                    .sortname
-                    .get(self.context.resultsort.get(id).unwrap())
-                    .unwrap()
-            ));
-        }
         for seq in self.sequents.iter() {
-            s.push_str(&format!("(assert {})", self.show_sequent(seq)));
+            s.push_str(&format!("{}\n", self.show_sequent(seq)));
         }
         return s;
     }
@@ -380,5 +362,6 @@ pub enum Error {
     Undeclared(String),
     AlreadyDeclared(String),
     DeclaredTwice(String),
+    IllegalTerm(String),
     IllegalSequent(String),
 }
